package wafly.framework.storyTK.basic;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

public class StoreMgr
{
	private String savePath = Config.iRootPath;
	private Properties params = new Properties();

	/**
	 * 通过存档路径初始化一个SL模块<br>
	 * init a SaveLoad module with the specified path
	 * 
	 * @param path
	 *            存档路径,不包括文件名<br>
	 *            the save file's directory
	 */
	public StoreMgr(String path)
	{
		savePath = Util.createFolder(path);
	}

	/**
	 * 添加或替换一对(名称-数值),其中数值名称(key)不可为null; 如果数值(value)为null则会移除已存在的一对key-value<br>
	 * put the key-value into, key could not be null; if the value is null, will remove the current
	 * key.
	 * 
	 * @param key
	 *            名称
	 * @param value
	 *            值
	 */
	public void setValue(String key, Object value)
	{
		if (key == null)
			return;
		if (value == null)
		{
			if (params.contains(value))
				params.remove(key);
			return;
		}
		params.setProperty(key, value.toString());
	}

	/**
	 * 添加或替换一组(名称-数值)<br>
	 * To put the HashMap into current key-value mapping. If one or more key exists, the old value
	 * is replaced.
	 * 
	 * @param params
	 *            一组(数值-名称)映射
	 */
	public void setValues(HashMap<String, String> params)
	{
		if (params == null)
			return;
		Iterator<String> keyIt = params.keySet().iterator();
		while (keyIt.hasNext())
		{
			String key = keyIt.next();
			String value = params.get(key);
			if (value == null)
				value = "";
			this.params.setProperty(key, value);
		}
	}

	/**
	 * 获取已添加的值(找不到值名称的时候返回null)<br>
	 * to get the value via exists key, return null if key not found
	 */
	public String getValue(String key)
	{
		String value = params.getProperty(key);
		try
		{
			if (value != null)
				value = new String(value.trim().getBytes("ISO-8859-1"), "UTF-8");
		} catch (Exception e)
		{
			Util.logError("Fail to get value.", e);
		}
		return value;
	}

	/**
	 * 获取时间(毫秒值),只有在调用了save()或者load()函数后才能获得有效值<br>
	 * to get the time stamp from save file, only used after save() or load()
	 * 
	 * @return 存档时间的毫秒数<br>
	 *         the game millisecond from save file, or return 0 if error in getting.
	 */
	public long getTimeStamp()
	{
		try
		{
			return Long.parseLong(params.getProperty("TIMESTAMP"));
		} catch (Exception e)
		{
			Util.logError("Fail to get timestamp.", e);
			return 0;
		}
	}

	/**
	 * 获取已添加的所有(名称-数值)序列<br>
	 * get all of the current key-value mapping
	 */
	public HashMap<String, String> getAll()
	{
		HashMap<String, String> values = new HashMap<String, String>();
		Iterator<Object> it = params.keySet().iterator();
		while (it.hasNext())
		{
			String key = (String) it.next();
			values.put(key, getValue(key));
		}
		return values;
	}

	/**
	 * 将当前已添加的所有(名称-数值)序列保存到文件, 需要存储卡写权限<br>
	 * save the key-value mapping to file
	 * 
	 * @param index
	 *            存档文件的序号<br>
	 *            save file index
	 * @return return 保存成功返回true, 否则返回false<br>
	 *         true if save successful, otherwise return false
	 */
	public boolean save(int index)
	{
		boolean result = true;
		File file = new File(savePath + index + ".sav");
		OutputStreamWriter writer = null;
		FileOutputStream fout = null;
		BufferedOutputStream bout = null;
		FileOutputStream ifout = null;
		Iterator<Object> keyIt = null;
		String key = null;
		try
		{
			if (!file.exists())
				file.createNewFile();
			fout = new FileOutputStream(file);
			writer = new OutputStreamWriter(fout, "UTF-8");
			writer.write("#Please do not change by manual.");
			writer.write("#\n");
			params.setProperty("TIMESTAMP", Long.toString(System.currentTimeMillis()));
			keyIt = params.keySet().iterator();
			while (keyIt.hasNext())
			{
				key = (String) (keyIt.next());
				writer.write(key + "=" + params.getProperty(key) + "\n");
			}
			writer.flush();

//			Arena.getScene().drawGamePicture(thumbnailDrawable);
//			thumbnailDrawable.saveToFile(savePath + index + ".jpg");
		} catch (Exception e)
		{
			Util.logError("Fail to save.", e);
			result = false;
		} finally
		{
			try
			{
				if (writer != null)
					writer.close();
				if (fout != null)
					fout.close();
				if (bout != null)
					bout.close();
				if (ifout != null)
					ifout.close();
			} catch (Exception e)
			{
				result = false;
			}
		}
		return result;
	}

	/**
	 * 从存档文件中读取(名称-数值)序列并添加到当前<br>
	 * to load the key-value mapping from file
	 * 
	 * @param index
	 *            从中读取数据的存档文件的序号<br>
	 *            save file index
	 * @return return 读取成功则返回true, 否则返回false<br>
	 *         true if save successful, otherwise return false
	 */
	public boolean load(int index)
	{
		boolean result = true;
		FileInputStream fis = null;
		try
		{
			params.clear();
			fis = new FileInputStream(savePath + index + ".sav");
			params.load(fis);
//			thumbnailDrawable = new DrawableHelper(savePath + index + ".jpg");
		} catch (Exception e) {
			result = false;
			Util.logError("Fail to load.", e);
		} finally {
			try {
				fis.close();
			} catch (Exception e) { }
		}

		return result;
	}

}
