package wafly.framework.storyTK.scene.war;

import java.util.ArrayList;
import java.util.HashMap;
import wafly.framework.storyTK.Scene;
import wafly.framework.storyTK.basic.DrawableHelper;
import wafly.framework.storyTK.basic.Util;
import wafly.framework.storyTK.control.Sprite;

public class WarScene extends Scene {
	
	private int standSize=0, collidedDistanse=0;
	private ArrayList<Sprite> spriteList = new ArrayList<Sprite>();
	private ArrayList<int[]> obstacleList = new ArrayList<int[]>();
	private HashMap<String, String> paramMap = new HashMap<String, String>();
	
	public WarScene(int standSize){
		this.standSize=standSize;
		collidedDistanse=(int) (2.42*standSize);
	}

	@Override
	public void onDraw(DrawableHelper draw) {
		for(int i=0; i<spriteList.size(); i++)
			spriteList.get(i).onDraw(draw);
	}
	
	public void addObstacle(int x, int y, int width, int height){
		this.obstacleList.add(new int[]{x, y, width, height});
	}
	
	public int getStandSize(){return standSize;}
	
	public void reOrderZ(Sprite sprite){
		if(!spriteList.contains(sprite))
			return;
		for(int i=0; i<spriteList.size(); i++){
			if(sprite.getStandAreaPositionY()>=spriteList.get(i).getStandAreaPositionY())
				continue;
			if(i>0 && spriteList.get(i-1)==sprite)
				break;
			synchronized(this){
				spriteList.remove(sprite);
				spriteList.add(i, sprite);
			}
			break;
		}
	}

	/**
	 * 收集与指定范围发生碰撞的精灵
	 * @param left 需要判断的中心（基于屏幕）
	 * @param top 需要判断的中心（基于屏幕）
	 * @param skipSprites 不需要检测的精灵
	 */
	public ArrayList<Sprite> getCollideSprites(int x, int y, Sprite...skipSprites){
		ArrayList<Sprite> collideSpriteList = new ArrayList<Sprite>();
		for(Sprite sp:spriteList){
			boolean isSkip=false;
			for(int j=0; j<skipSprites.length; j++){
				if(sp==skipSprites[j]){
					isSkip=true;
					break;
				}
			}
			if(isSkip)
				continue;
			if(Math.abs(x-sp.getX()-sp.getStandAreaPositionX()) + 
					Math.abs(y-sp.getY()-sp.getStandAreaPositionY()) < collidedDistanse)
				collideSpriteList.add(sp);
		}
		return collideSpriteList;
	}
	
	/**
	 * 判断指定的范围是否与现有精灵发生碰撞
	 * @param left 需要判断的范围（基于屏幕）
	 * @param top 需要判断的范围（基于屏幕）
	 * @param skipSprites 不需要检测的精灵
	 */
	public boolean isCollidedBySprite(int x, int y, Sprite...skipSprites){
		for(Sprite sp:spriteList){
			boolean isSkip=false;
			for(int j=0; j<skipSprites.length; j++){
				if(sp==skipSprites[j]){
					isSkip=true;
					break;
				}
			}
			if(isSkip)
				continue;
			if(Math.abs(x-sp.getX()-sp.getStandAreaPositionX()) + 
					Math.abs(y-sp.getY()-sp.getStandAreaPositionY()) < collidedDistanse)
				return true;
		}
		return false;
	}
	
	public boolean isCollidedByObstacle(int x, int y){
		for(int[] area:obstacleList){
			if(x+standSize>area[0]&&y+standSize>area[1]&&x-standSize<area[2]&&y-standSize<area[3])
				return true;
		}
		return false;
	}
	
	public synchronized void addSprite(Sprite sprite){
		boolean done = false;
		for(int i=0; i<spriteList.size(); i++){
			if(sprite.getStandAreaPositionY()>=spriteList.get(i).getStandAreaPositionY())
				continue;
			spriteList.add(i, sprite);
			done=true;
			break;
		}
		if(!done)
			spriteList.add(sprite);
		sprite.setScene(this);
	}
	
	/**从精灵族中移除指定的精灵；注意：这个方法会将精灵的组属性重置为一个空组*/
	public void removeSprite(Sprite sprite){
		spriteList.remove(sprite);
		sprite.setScene(this);
	}
	
	/**通过精灵ID在当前组内查找指定的精灵，在多个精灵拥有同样ID的情况下，返回第一个找到的拥有指定ID的精灵*/
	public Sprite getSprite(String spriteID){
		if(spriteID==null)
			return null;
		for(Sprite sp : spriteList){
			if(spriteID.equals(sp.getRoleID()))
				return sp;
		}
		return null;
	}
	
	/**获取当前场景中的所有精灵*/
	public ArrayList<Sprite> getAllSprites(){ return spriteList; }
	
	/**设置一个环境参数*/
	public void setParam(String key, Object value){
		this.paramMap.put(key, value.toString());
	}
	
	/**获取一个指定的环境参数的值*/
	public String getParam(String key){
		return paramMap.get(key);
	}
	
//	/**获取两个精灵之间的距离*/
//	public int getDistance(Sprite sprite1, Sprite sprite2){
//		int dWidth = (sprite1.getX()+sprite1.getStandAreaPositionX()-sprite2.getX()-sprite2.getStandAreaPositionX()-standSize)
//				* (sprite1.getX()+sprite1.getStandAreaPositionX()-sprite2.getX()-sprite2.getStandAreaPositionX()-standSize);
//		int dHeight = (sprite1.getY()+sprite1.getStandAreaPositionY()-sprite2.getY()-sprite2.getStandAreaPositionY()-standSize)
//				* (sprite1.getY()+sprite1.getStandAreaPositionY()-sprite2.getY()-sprite2.getStandAreaPositionY()-standSize);
//		return (int)Math.sqrt(dWidth+dHeight);
//	}

	@Override
	public void destroy() {
		for(Sprite sp : spriteList)
			sp.destroy();
	}
	
	public void loadMap(String mapPath, int startLocation_X, int startLocation_Y) {
		DrawableHelper map = new DrawableHelper(mapPath);
		if (map.getWidth() > Util.getGameWidth())
		{
			if (startLocation_X < 0)
			{
				startLocation_X = 0;
			}
			else if (Util.getGameWidth() + startLocation_X > map.getWidth())
			{
				startLocation_X = map.getWidth() - Util.getGameWidth();
			}
		}
		if (map.getHeight() > Util.getGameHeight())
		{
			if (startLocation_Y < 0)
			{
				startLocation_Y = 0;
			}
			else if (Util.getGameHeight() + startLocation_Y > map.getHeight())
			{
				startLocation_Y = map.getHeight() - Util.getGameHeight();
			}
		}
		setBackground(map, -startLocation_X, -startLocation_Y, null);
	}

	private int tx=0,ty=0;
	@Override
	public void onUserTouchScreen(int x, int y) {
		tx=x;
		ty=y;
	}

	@Override
	public void onUserMoveScreen(int x, int y) {
		int disx=x-tx;
		int disy=y-ty;
		moveBackground(disx+getBackgroundLocation_X(), disy+getBackgroundLocation_Y());
		for(Sprite sp:spriteList)
			sp.setLocation(sp.getX()+disx, sp.getY()+disy);
	}

}
